/*
  SetElevation.cpp

  Implementation of the SetElevation function
*/

#include <stdio.h>
#include <stdlib.h>

#include "TGAFile.h"

/*
void SetElevationScaled(const int x,
                        const int y,
                        const double height)

set the pixel value as a three byte signed integer

Parameters:
x                      const int       x coordinate of pixel
                                       requested
y                      const int       y coordinate of pixel
                                       requested
height                 const double    the elevation to set

Return value: none

Global variables affected: none
*/

void TGAFile::SetElevationScaled(const int x, const int y, const double height)
{
  if(x >= 0 && x < m_pHeader->image_width && y >= 0 && y < m_pHeader->image_height)
  {
    unsigned char* pPos1;
    unsigned char* pPos2;
    int i;
    long lHeight;
    
    if(m_zScale)
    {
      lHeight = long(height / m_zScale + 0.5);
    }
    else
    {
      lHeight = -1;
    }

    pPos1 = (unsigned char *)&lHeight;
    pPos2 = m_pData + y * m_dwWidthDWORDAligned + x * m_nBytesPerPixel;
    // Put into memory.
    for(i = 0; i < m_nBytesPerPixel; i++)
    {
      *pPos2++ = *pPos1++; /* BLUE, GREEN, RED [, ALPHA]  */
    }
  }
}

/*
void SetElevationUnscaled(const int x,
                          const int y,
                          const long height)

set the pixel value as a three byte signed integer

Parameters:
x                      const int       x coordinate of pixel
                                       requested
y                      const int       y coordinate of pixel
                                       requested
height                 const long      the elevation to set

Return value: none

Global variables affected: none
*/

void TGAFile::SetElevationUnscaled(const int x, const int y, const long height)
{
  if(x >= 0 && x < m_pHeader->image_width && y >= 0 && y < m_pHeader->image_height)
  {
    unsigned char* pPos1;
    unsigned char* pPos2;
    int i;
    
    pPos1 = (unsigned char *)&height;
    pPos2 = m_pData + y * m_dwWidthDWORDAligned + x * m_nBytesPerPixel;
    // Put into memory.
    for(i = 0; i < m_nBytesPerPixel; i++)
    {
      *pPos2++ = *pPos1++; /* BLUE, GREEN, RED [, ALPHA]  */
    }
  }
}
