/*
  Write.cpp

  Implementation of the Write function.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <io.h>
#include <fcntl.h>

#include "TGAFile.h"

/*
int Write(const char *szFilename)

Save TGA file

Parameters:
szFilename           const char*     Name of output file

Return value:
0 if no error occurred
a non-zero value if an error occurred
*/

int TGAFile::Write(const char* szFilename)
{
  long lWidthInBytes;
  unsigned char *pPixels;
  size_t lResult;
  int y;
  FILE *pOutputfile;

  if(!strcmp(szFilename, "-"))
  {
    // The filename is "-" ... this is stdout.
    // Set "stdout" to have binary mode:
    int nResult = _setmode(_fileno(stdout), _O_BINARY);
    if(nResult == -1)
    {
      fprintf(stderr, "TGAFile::Open(): Cannot set mode _O_BINARY for stdin.\n");
      return 1;
    }
    pOutputfile = stdout;
  }
  else
  {
    pOutputfile = fopen(szFilename, "wb");
    if(!pOutputfile)
    {
      fprintf(stderr, "TGAFile::Write(): Cannot open file \"%s\".\n", szFilename);
      return 2;
    }
  }
  m_pHeader->image_discr.OrgBit = 0;
  fwrite(m_pHeader,
         1,
         sizeof(TGAFILEHEADER),
         pOutputfile);

  lWidthInBytes = (m_pHeader->pixel_size / 8) * m_pHeader->image_width;
  pPixels = m_pData;
  for(y = 0;
      y < m_pHeader->image_height;
      y++)
  {
    lResult = fwrite(pPixels,
                     1,
                     lWidthInBytes,
                     pOutputfile);
    pPixels += m_dwWidthDWORDAligned;
  }

  fclose(pOutputfile);
  return 0;
}
