// AddWaypoints.cpp: implementation file

#include "stdafx.h"

#include <afxtempl.h>
#include <float.h>

#include <list>
#include "Waypoint.h"

#include "geoDataClient.h"
#include "geoDataClientDialog.h"
#include "HTTPRequest.h"

// Add waypoints to a TGA file. Use a predefined waypoint list.
// Returns 0 if OK.
int CgeoDataClientDialog::AddWaypoints(const CString &tgaFilenameTextureCombined,
                                       const CString &tgaFilenameTextureCombinedWaypoints,
                                       double left,
                                       double top,
                                       double right,
                                       double bottom,
                                       std::list<waypoint> waypointList)
{
  CString message;
  TGAFile tgaFileCombined;
  TGAFile tgaFileCombinedWaypoints;

  if(tgaFileCombined.Open(tgaFilenameTextureCombined))
  {
    message.Format(_T("Error in AddWaypoints(): Could not open file \"%s\"."), tgaFilenameTextureCombined);
    addMessage(message);
    return 1;
  }
  int nWidth  = tgaFileCombined.GetImageWidth();
  int nHeight = tgaFileCombined.GetImageHeight();
  RGBQUAD Color;

  CDC MemoryDC;
  MemoryDC.CreateCompatibleDC(NULL);
  CBitmap MaskBitmap;
  BOOL bResult = MaskBitmap.CreateBitmap(nWidth, nHeight, 1, 1, NULL);
  CPen MaskPen(PS_SOLID, lineThickness, RGB(255, 255, 255)); // Create a solid red pen of width 5.
//  MemoryDC.SetBkColor(RGB(255, 255, 255));
//  MemoryDC.SetBkMode(TRANSPARENT);
  CBitmap* pOldBitmap = MemoryDC.SelectObject(&MaskBitmap);
  CPen* pOldPen = MemoryDC.SelectObject(&MaskPen);

  std::list<waypoint>::iterator waypointIterator;
  waypointIterator = waypointList.begin();

  while(waypointIterator != waypointList.end())
  {
    // The geographic coordinates.
    double Xg = waypointIterator->longitude;
    double Yg = waypointIterator->latitude;
    // The pixel coordinates -> the georeferenced waypoint.
    // Explanation for the " - 0.5 + 0.5" in the next lines:
    // The + 0.5 comes from the rounding in the MoveTo() and LineTo() call below.
    double Xp = (Xg - left) / (right - left) * nWidth - 0.5 + 0.5;
    double Yp = (Yg - bottom) / (top - bottom) * nHeight - 0.5 + 0.5;
    if(waypointIterator == waypointList.begin())
    {
      MemoryDC.MoveTo(int(Xp), int(Yp));
  #ifdef _DEBUG
      {
        CString message;
        message.Format(_T("DEBUG: MoveTo(%f/%f)"), Xp, Yp);
        addMessage(message);
      }
  #endif // _DEBUG
    }
    else
    {
      MemoryDC.LineTo(int(Xp), int(Yp));
  #ifdef _DEBUG
      {
        CString message;
        message.Format(_T("DEBUG: LineTo(%f/%f)"), Xp, Yp);
        addMessage(message);
      }
  #endif // _DEBUG
    }
    waypointIterator++;
  }

  RGBQUAD LineColor;
  LineColor.rgbRed = 255;
  LineColor.rgbGreen = 0;
  LineColor.rgbBlue = 0;
  // Transfer the memory DC to the bitmap.
  tgaFileCombinedWaypoints.New(nWidth, nHeight);
  for(int y = 0; y < nHeight; y++)
  {
    for(int x = 0; x < nWidth; x++)
    {
      tgaFileCombined.GetColor(x, y, &Color);
      COLORREF C = MemoryDC.GetPixel(x, y);
      if(GetRValue(C))
      {
        tgaFileCombinedWaypoints.SetColor(x, y, &LineColor);
      }
      else
      {
        tgaFileCombinedWaypoints.SetColor(x, y, &Color);
      }
    }
  }
  tgaFileCombined.Close();

  MemoryDC.SelectObject(pOldBitmap); // New Adamiker 5/20/97
  MemoryDC.SelectObject(pOldPen);
  MaskBitmap.DeleteObject();
  MemoryDC.DeleteDC();

  if(tgaFileCombinedWaypoints.Write(tgaFilenameTextureCombinedWaypoints))
  {
    tgaFileCombinedWaypoints.Close();
    message.Format(_T("Error in AddWaypoints(): Could not write file \"%s\"."), tgaFilenameTextureCombinedWaypoints);
    addMessage(message);
    return 3;
  }
  tgaFileCombinedWaypoints.Close();
  message.Format(_T("Adding Waypoints successful: \"%s\" written."),
                 tgaFilenameTextureCombinedWaypoints);
  addMessage(message);
  return 0;
}
