// CombineTga.cpp: implementation file

#include "stdafx.h"

#include <afxtempl.h>
#include <float.h>

#include <list>
#include "Waypoint.h"

#include "geoDataClient.h"
#include "geoDataClientDialog.h"
#include "HTTPRequest.h"

// Combine two TGA files. Use the size of the first one.
// The white pixels of the first one are considered fully transparent.
int CgeoDataClientDialog::CombineTga(const CString &tgaFilename1,
                                     const CString &tgaFilename2,
                                     const CString &tgaFilenameCombined)
{
  CString message;
  message.Format(_T("Combining \"%s\" and \"%s\" to \"%s\"."),
                 tgaFilename1,
                 tgaFilename2,
                 tgaFilenameCombined);
  addMessage(message);

  TGAFile tgaFile1;
  TGAFile tgaFile2;
  TGAFile tgaFileCombined;

  if(tgaFile1.Open(tgaFilename1))
  {
    message.Format(_T("Error in CombineTga(): Could not open file \"%s\"."), tgaFilename1);
    addMessage(message);
    return 1;
  }
  if(tgaFile2.Open(tgaFilename2))
  {
    tgaFile1.Close();
    message.Format(_T("Error in CombineTga(): Could not open file \"%s\"."), tgaFilename2);
    addMessage(message);
    return 2;
  }
  CString tgaFilename2Stretched;
  int nWidth1  = tgaFile1.GetImageWidth();
  int nHeight1 = tgaFile1.GetImageHeight();
  int nWidth2  = tgaFile2.GetImageWidth();
  int nHeight2 = tgaFile2.GetImageHeight();
  if((nWidth2 != nWidth1) || (nHeight2 != nHeight1))
  {
    // We have to stretch/shrink tgaFile2.
    tgaFile2.Close();
    tgaFilename2Stretched.Format("%s.stretched.tga", tgaFilename2);
    int nResult;
    nResult = StretchTga(tgaFilename2,
                         tgaFilename2Stretched,
                         0,       0,
                         nWidth2, 0,
                         0,       nHeight2,
                         nWidth2, nHeight2,
                         nWidth1, nHeight1);
    if(tgaFile2.Open(tgaFilename2Stretched))
    {
      tgaFile1.Close();
      message.Format(_T("Error in CombineTga(): Could not open file \"%s\"."), tgaFilename2Stretched);
      return 2;
    }
    nWidth2  = tgaFile2.GetImageWidth();
    nHeight2 = tgaFile2.GetImageHeight();
  }
  RGBQUAD Color1;
  unsigned char *pBlue1 = (unsigned char *)&Color1;
  unsigned char *pGreen1 = pBlue1 + 1;
  unsigned char *pRed1 = pGreen1 + 1;
  RGBQUAD Color2;
  unsigned char *pBlue2 = (unsigned char *)&Color2;
  unsigned char *pGreen2 = pBlue2 + 1;
  unsigned char *pRed2 = pGreen2 + 1;
  RGBQUAD Color3;
  unsigned char *pBlue3 = (unsigned char *)&Color3;
  unsigned char *pGreen3 = pBlue3 + 1;
  unsigned char *pRed3 = pGreen3 + 1;
  const double dFactor1 = 0.0;
  const double dFactor2 = 0.0;
  double dValue;
  tgaFileCombined.New(nWidth1, nHeight1);

  // Get the list of transparent and lightening colors.
  CList<RGBQUAD, RGBQUAD&> transparentColorList;
  CList<RGBQUAD, RGBQUAD&> lighteningColorList;
  TGAFile legendFile;
  if(!legendFile.Open("legend.tga"))
  {
    message.Format(_T("CombineTga(): Legend file \"legend.tga\" present and loaded."));
    addMessage(message);
	message.Format(_T("CombineTga(): Using the following colors to define transparency and semi-transparency in TIGER maps:"));
    addMessage(message);
    for(int x = 0; x < legendFile.GetImageWidth(); x++)
    {
      RGBQUAD Color;
      // Row 1 (the top row) in the legend file contains a list of colors to be used as transparent.
      legendFile.GetColor(x, 1, &Color);
      POSITION pos = transparentColorList.GetHeadPosition();
      for(int i = 0; i < transparentColorList.GetCount(); i++)
      {
        RGBQUAD C = transparentColorList.GetNext(pos);
        if((C.rgbBlue == Color.rgbBlue) && (C.rgbGreen == Color.rgbGreen) && (C.rgbRed == Color.rgbRed))
        {
          // The color is already in the list.
          break;
        }
      }
      if(i == transparentColorList.GetCount())
      {
        // Was not in the list: add.
        transparentColorList.AddHead(Color);
      }
      // Row 0 (the bottom row) in the legend file contains a list of colors to be used as lightening colors.
      legendFile.GetColor(x, 0, &Color);
      pos = lighteningColorList.GetHeadPosition();
      for(int i = 0; i < lighteningColorList.GetCount(); i++)
      {
        RGBQUAD C = lighteningColorList.GetNext(pos);
        if((C.rgbBlue == Color.rgbBlue) && (C.rgbGreen == Color.rgbGreen) && (C.rgbRed == Color.rgbRed))
        {
          // The color is already in the list.
          break;
        }
      }
      if(i == lighteningColorList.GetCount())
      {
        // Was not in the list: add.
        lighteningColorList.AddHead(Color);
      }
    }
    legendFile.Close();
	POSITION pos;
	pos = transparentColorList.GetHeadPosition();
	for(int i = 0; i < transparentColorList.GetCount(); i++)
	{
		RGBQUAD C = transparentColorList.GetNext(pos);
		message.Format(_T("CombineTga(): transparentColor[%d]: (%d/%d/%d)"), i, C.rgbRed, C.rgbGreen, C.rgbBlue);
		addMessage(message);
	}
	pos = lighteningColorList.GetHeadPosition();
	for(int i = 0; i < lighteningColorList.GetCount(); i++)
	{
		RGBQUAD C = lighteningColorList.GetNext(pos);
		message.Format(_T("CombineTga(): lighteningColor[%d]: (%d/%d/%d)"), i, C.rgbRed, C.rgbGreen, C.rgbBlue);
		addMessage(message);
	}
  }
  else
  {
    // No legend file present.
    message.Format(_T("CombineTga(): No legend file \"legend.tga\" present."));
    addMessage(message);
	message.Format(_T("CombineTga(): Using the following colors to define transparency and semi-transparency in TIGER maps:"));
    addMessage(message);
    RGBQUAD Color;

    Color.rgbRed = 255; Color.rgbGreen = 255; Color.rgbBlue = 255; // White.
	message.Format(_T("CombineTga():     transparentColor[0]: (255/255/255), white"));
    addMessage(message);
    transparentColorList.AddHead(Color);

    Color.rgbRed = 255; Color.rgbGreen = 180; Color.rgbBlue = 170; // Pink (military area).
	message.Format(_T("CombineTga():     lighteningColor[0]: (255/180/170), pink (military area)"));
    addMessage(message);
    lighteningColorList.AddHead(Color);
    Color.rgbRed = 175; Color.rgbGreen = 215; Color.rgbBlue = 127; // Dark green (national park).
	message.Format(_T("CombineTga():     lighteningColor[1]: (175/215/127), dark green (national park)"));
    addMessage(message);
    lighteningColorList.AddHead(Color);
    Color.rgbRed = 127; Color.rgbGreen = 255; Color.rgbBlue = 127; // Light green (other park).
	message.Format(_T("CombineTga():     lighteningColor[2]: (127/255/127), dight green (other park)"));
    addMessage(message);
    lighteningColorList.AddHead(Color);
    Color.rgbRed = 255; Color.rgbGreen = 247; Color.rgbBlue = 159; // Yellow (city).
	message.Format(_T("CombineTga():     lighteningColor[3]: (255/247/159), yellow (city)"));
    addMessage(message);
    lighteningColorList.AddHead(Color);
  }
  bool colorSet;
  for(int y = 0; y < nHeight1; y++)
  {
    for(int x = 0; x < nWidth1; x++)
    {
      colorSet = false;
      tgaFile1.GetColor(x, y, &Color1);
      POSITION pos;
      pos = transparentColorList.GetHeadPosition();
      for(int i = 0; i < transparentColorList.GetCount(); i++)
      {
        RGBQUAD C = transparentColorList.GetNext(pos);
        if((C.rgbBlue == Color1.rgbBlue) && (C.rgbGreen == Color1.rgbGreen) && (C.rgbRed == Color1.rgbRed))
        {
          // Color found.
          // Use only the color of the second pixel.
          tgaFile2.GetColor(x, y, &Color2);
          tgaFileCombined.SetColor(x, y, &Color2);
          colorSet = true;
          break;
        }
      }
      if(colorSet)
      {
        continue;
      }
      pos = lighteningColorList.GetHeadPosition();
      for(int i = 0; i < lighteningColorList.GetCount(); i++)
      {
        RGBQUAD C = lighteningColorList.GetNext(pos);
        if((C.rgbBlue == Color1.rgbBlue) && (C.rgbGreen == Color1.rgbGreen) && (C.rgbRed == Color1.rgbRed))
        {
          // Color found.
          // Merge a little lighter.
            Color1.rgbRed += (255 -   Color1.rgbRed) / 2;
          Color1.rgbGreen += (255 - Color1.rgbGreen) / 2;
           Color1.rgbBlue += (255 -  Color1.rgbBlue) / 2;
          tgaFile2.GetColor(x, y, &Color2);
          dValue = (double(*pRed1 + dFactor1) * double(*pRed2 + dFactor2)) / 255.0 + 0.5;
          if(dValue > 255.0) dValue = 255.0; if(dValue < 0.0) dValue = 0.0;
          *pRed3 = int(dValue);
          dValue = (double(*pGreen1 + dFactor1) * double(*pGreen2 + dFactor2)) / 255.0 + 0.5;
          if(dValue > 255.0) dValue = 255.0; if(dValue < 0.0) dValue = 0.0;
          *pGreen3 = int(dValue);
          dValue = (double(*pBlue1 + dFactor1) * double(*pBlue2 + dFactor2)) / 255.0 + 0.5;
          if(dValue > 255.0) dValue = 255.0; if(dValue < 0.0) dValue = 0.0;
          *pBlue3 = int(dValue);
          tgaFileCombined.SetColor(x, y, &Color3);
          colorSet = true;
          break;
        }
      }
      if(colorSet)
      {
        continue;
      }
      // The pixel in the first file has a thematic color (lake/pond/ocean/road/...).
      // Use only that color.
      tgaFileCombined.SetColor(x, y, &Color1);
    }
  }
  if(!tgaFilename2Stretched.IsEmpty())
  {
    // We had to stretch/shrink tgaFile2.
    if(removeIntermediateFiles)
    {
      DeleteFile(tgaFilename2Stretched);
    }
  }
  tgaFile2.Close();
  tgaFile1.Close();

  // For the final or licensed version, this line has to be removed.
#if 0
  if(WriteDemoTga(tgaFileCombined))
  {
    message.Format(_T("Error in CombineTga(): Could not stamp as demo version."));
    addMessage(message);
    return 4;
  }
#endif

  if(tgaFileCombined.Write(tgaFilenameCombined))
  {
    message.Format(_T("Error in CombineTga(): Could not write file \"%s\"."), tgaFilenameCombined);
    addMessage(message);
    return 3;
  }
  message.Format(_T("Combining successful: \"%s\" written."),
                 tgaFilenameCombined);
  addMessage(message);
  return 0;
}
