// ElevationToColor.cpp: implementation file

#include "stdafx.h"

#include <afxtempl.h>
#include <float.h>

#include <list>
#include "Waypoint.h"

#include "geoDataClient.h"
#include "geoDataClientDialog.h"
#include "HTTPRequest.h"
#include "Palette.h"

// Convert elevation to color using a palette [file].
// Returns 0 if OK.
int CgeoDataClientDialog::ElevationToColor(const CString &tgaFilenameElevation,
                                           const CString &tgaFilenameElevationColor)
{
  CString message;
  message.Format(_T("Converting elevation \"%s\" to color \"%s\"."),
                 tgaFilenameElevation,
                 tgaFilenameElevationColor);
  addMessage(message);

  TGAFile inputFile;
  if(!inputFile.Open(tgaFilenameElevation))
  {
    TGAFile outputFile;
    RGBQUAD Color;
    Color.rgbReserved = 0;
    outputFile.New(inputFile.GetImageWidth(),
                   inputFile.GetImageHeight());
    for(int y = 0; y < inputFile.GetImageHeight(); y++)
    {
      for(int x = 0; x < inputFile.GetImageWidth(); x++)
      {
        double elevation = inputFile.GetElevationScaled(x, y);
        // 8000 meters -> 2000 pixels.
        int value = int(elevation * ColorPaletteSize / 8000.0);
        if(value >= ColorPaletteSize)
        {
          value = ColorPaletteSize - 1;
        }
        if(value < 0)
        {
          value = 0;
        }
        Color.rgbRed   = ColorPalette[value][0];
        Color.rgbGreen = ColorPalette[value][1];
        Color.rgbBlue  = ColorPalette[value][2];
        outputFile.SetColor(x, y, &Color);
      }
    }
    inputFile.Close();
    if(outputFile.Write(tgaFilenameElevationColor))
    {
      message.Format(_T("Error in ElevationToColor(): Could not write \"%s\"."),
                     tgaFilenameElevationColor);
      addMessage(message);
      outputFile.Close();
      return 1;
    }
    else
    {
      outputFile.Close();
      message.Format(_T("Color conversion successful: \"%s\" written."),
                     tgaFilenameElevationColor);
      addMessage(message);
      return 0;
    }
  }
  else
  {
    message.Format(_T("Error in ElevationToColor(): Could not read \"%s\"."),
                    tgaFilenameElevation);
    addMessage(message);
    return 2;
  }
}
