// ReadWaypoints.cpp: implementation file

#include "stdafx.h"

#include <afxtempl.h>
#include <float.h>

#include <list>
#include "Waypoint.h"

#include "geoDataClient.h"
#include "geoDataClientDialog.h"
#include "HTTPRequest.h"

/*
Open the given file and read waypoints.
This function considers itself OK if it could read at least one waypoint.
A waypoint file line contains at least two white space separated entries: latitude and longitude.

latitude longitude comment

Example:

# A hash sign in the first column indicates a comment line.
34.567   112.345   The rest of a line after column 2 is considered comment or waypoint name.
34.568   112.346
*/

bool CgeoDataClientDialog::readWaypoints(const CString &pathName)
{
  waypointList.clear();

  FILE *waypointFile;
  waypointFile = fopen(pathName, "r");
  if(!waypointFile)
  {
    CString message;
    message.Format("Error: File \"%s\" could not be opened.", pathName);
    AfxMessageBox(message, MB_ICONSTOP);
    return true;
  }
  char szLine[1024];
  fgets(szLine, sizeof(szLine), waypointFile);
  while(!feof(waypointFile))
  {
    double latitude = 0.0;
    double longitude = 0.0;
    // Test if it's a comment line.
    if(szLine[0] != '#')
    {
      char *szLatitude = strtok(szLine, " \t"); // Read the first column.
      if(szLatitude)
      {
        latitude = atof(szLatitude);
        char *szLongitude = strtok(NULL, " \t"); // Read the second column.
        if(szLongitude)
        {
          longitude = atof(szLongitude);
          char *szComment = strtok(NULL, "\n"); // This time the delimiter is the newline character!
          if(szComment)
          {
            while(*szComment && isspace(*szComment))
            {
              szComment++;
            }
          }
          waypoint w(latitude, longitude, szComment);
          waypointList.push_back(w);
        }
      }
    }
    fgets(szLine, sizeof(szLine), waypointFile);
  }
  if(waypointList.size() == 0)
  {
    CString message;
    message.Format("Error: No waypoints in the file\n"
                   "\"%s\".\n\n"
                   "Expected layout of a line:\n\n"
                   "latitude longitude [comment]", pathName);
    AfxMessageBox(message, MB_ICONSTOP);
    return true;
  }
  fclose(waypointFile);
  return false;
}
